/*******************************************************************************
 * Copyright (C) Altimetrik 2018. All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Altimetrik. You shall not disclose such Confidential Information
 * and shall use it only in accordance with the terms and conditions
 * entered into with Altimetrik.
 ******************************************************************************/
package com.altimetrik.playground.innovation.exception;

import java.util.Locale;

import com.altimetrik.playground.innovation.util.PlatformErrorConstant;

public abstract class PgApplicationException extends Exception {

	private static final long serialVersionUID = 1186927232091885741L;
	private static final String ERR_PREFIX = "[Error Code: ";
	private static final String ERR_SUFFIX = "] ";
	private final Object[] errorArguments;

	public PgApplicationException() {
		this(PlatformErrorConstant.APPLICATION_ERROR);
	}

	/**
	 * single argument constructor.
	 * @param errorCode - this argument receives error code instead of message.
	 * @param errorArgs - the arguments to be used to replace values within the
	 *            message.
	 */
	public PgApplicationException(String errorCode, Object... errorArgs) {
		this(errorCode, null, errorArgs);
	}

	/**
	 * Preferred constructor to be used to wrap the underlying exception within this
	 * exception object.
	 * @param errorCode - object encapsulating the error details(error code and
	 *            message) with the predefined errorCode and message template.
	 * @param cause - the underlying cause of the exception
	 * @param errorArgs - the arguments to be used to replace values within the
	 *            message.
	 */
	public PgApplicationException(String errorCode, Throwable cause, Object... errorArgs) {
		super(errorCode, cause);
		this.errorArguments = errorArgs;
	}

	/**
	 * Constructor with only exception argument.
	 * @param cause - passes exception instance.
	 */
	public PgApplicationException(Throwable cause) {
		this(PlatformErrorConstant.APPLICATION_ERROR, cause);
	}

	/**
	 * Constructor with error code and exception instance and other attribute.
	 * @param errorCode - this argument receives error code instead of message.
	 * @param cause - instance of any exception
	 * @param enableSuppression - enable suppression
	 * @param writableStackTrace - writable stack trace.
	 * @param errorArgs - the arguments to be used to replace values within the
	 *            message.
	 */
	public PgApplicationException(String errorCode, Throwable cause, boolean enableSuppression, boolean writableStackTrace, Object... errorArgs) {
		super(errorCode, cause, enableSuppression, writableStackTrace);
		this.errorArguments = errorArgs;
	}

	/**
	 * Subclasses to provide their implementation to get the localized message for
	 * the given error code.
	 * @param errorCode - to translate into localized message.
	 * @param locale - locale to identify the messages.
	 * @param args - to replace with the place holder if any.
	 * @return - returns the localized text.
	 */
	public abstract String getLocalizedMessage(String errorCode, Locale locale, Object... args);

	/**
	 * The component name for this exception.
	 * @return the component name of the module to which the exception is associated
	 *         with.
	 */
	private String getComponent() {
		return this.getClass().getSimpleName();
	}

	public Object[] getArguments() {
		return errorArguments;
	}

	public String getUserMessage() {
		return super.getMessage();
	}

	public String getTraceabilityMessage() {
		String message = super.getMessage();
		StringBuilder sb = new StringBuilder();
		sb.append(getComponent()).append(" : ");
		String errCode = message;
		if (errCode != null) {
			sb.append(ERR_PREFIX);
			sb.append(errCode);
			sb.append(ERR_SUFFIX);
		}

		sb.append(buildMessage(message, getCause()));

		message = sb.toString();
		return message;
	}

	/**
	 * Builds a message for the given base message and root cause.
	 * @param message the base message
	 * @param cause the root cause
	 * @return the full exception message
	 */
	private static String buildMessage(String message, Throwable cause) {
		if (cause != null) {
			StringBuilder sb = new StringBuilder();
			if (message != null) {
				sb.append(message).append("; ");
			}
			sb.append("nested exception is ").append(cause);
			return sb.toString();
		}
		return message;
	}
}
