/**
 * Copyright (C) Altimetrik 2018. All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Altimetrik. You shall not disclose such Confidential Information
 * and shall use it only in accordance with the terms and conditions
 * entered into with Altimetrik.
 */

package com.altimetrik.playground.innovation.security;

import java.io.IOException;
import java.util.Base64;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.web.authentication.AnonymousAuthenticationFilter;

import com.altimetrik.playground.innovation.bean.Principal;
import com.fasterxml.jackson.databind.ObjectMapper;

import lombok.extern.slf4j.Slf4j;

/**
 * Detects if there is no {@code Authentication} object in the
 * {@code SecurityContextHolder}, and populates it with gateway authenticated
 * user details.
 *
 * @author Altimetrik CodeGen.
 */
@Slf4j
public class GatewayAuthenticationTokenFilter extends AnonymousAuthenticationFilter {

	@Autowired
	private AuthConfig authConfig;

	/**
	 * Instantiates a new gateway authentication filter.
	 * @param key the key
	 */
	public GatewayAuthenticationTokenFilter(String key) {
		super(key);
	}

	/**
	 * Creates the authentication.
	 * @param request the request
	 * @return the authentication
	 */
	@Override
	protected Authentication createAuthentication(HttpServletRequest request) {
		Principal principal = null;
		String authToken = StringUtils.defaultIfBlank(request.getHeader(AuthConfig.X_AUTHENTICATION_TOKEN), null);
		if (!org.springframework.util.StringUtils.isEmpty(authToken)) {
			try {
				byte[] base64AuthToken = Base64.getDecoder().decode(authToken);
				principal = new ObjectMapper().readValue(new String(base64AuthToken), Principal.class);
			}
			catch (IOException e) {
				log.error("Failed to parse principal object from auth token: '" + authToken + "'", e);
				principal = null;
			}
		}

		if (principal == null) {
			principal = new Principal();
			principal.setUserName(authConfig.getDefaultUser());
			principal.setUserId(authConfig.getDefaultUserId());
			principal.setUserInfoMstrId(Long.toString(authConfig.getUserInfoMstrId()));
			principal.setRealm(authConfig.getDefaultRealm());
			principal.setRoles(authConfig.getDefaultRoles());
			principal.setAuthenticated(Boolean.TRUE);
		}

		List<GrantedAuthority> authorities = AuthorityUtils.commaSeparatedStringToAuthorityList(principal.getRoles());
		AnonymousAuthenticationToken authentication = new AnonymousAuthenticationToken(principal.getRealm(), principal, authorities);
		authentication.setAuthenticated(principal.getAuthenticated());

		if (log.isDebugEnabled()) {
			log.debug("Populated SecurityContextHolder with gateway authenticated user: '" + authentication + "'");
		}

		return authentication;
	}

}
