/**
 * Copyright (C) Altimetrik 2018. All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Altimetrik. You shall not disclose such Confidential Information
 * and shall use it only in accordance with the terms and conditions
 * entered into with Altimetrik.
 */

package com.altimetrik.playground.innovation.security;

import javax.servlet.http.HttpServletResponse;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AnonymousAuthenticationProvider;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.AuthenticationTrustResolver;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.authentication.AnonymousAuthenticationFilter;

/**
 * Security config class.
 *
 * @author Altimetrik Technologies.
 */
@Configuration
@EnableWebSecurity
@EnableGlobalMethodSecurity(prePostEnabled = true)
public class SecurityConfig extends WebSecurityConfigurerAdapter {

	/** The regulus. */
	private static String strRegulus = "regulus";

	@Override
	protected void configure(AuthenticationManagerBuilder auth) throws Exception {
		auth.authenticationProvider(gatewayAuthenticationProvider());
	}

	/**
	 * Gateway authentication provider.
	 *
	 * @return the authentication provider
	 */
	@Bean
	public AuthenticationProvider gatewayAuthenticationProvider() {
		return new AnonymousAuthenticationProvider(strRegulus);
	}

	/**
	 * Gateway authentication filter.
	 *
	 * @return the anonymous authentication filter
	 */
	@Bean
	public AnonymousAuthenticationFilter gatewayAuthenticationFilter() {
		return new GatewayAuthenticationFilter(strRegulus);
	}

	/**
	 * Gateway authentication token filter.
	 *
	 * @return the anonymous authentication filter
	 */
	@Bean
	public AnonymousAuthenticationFilter gatewayAuthenticationTokenFilter() {
		return new GatewayAuthenticationTokenFilter(strRegulus);
	}

	/**
	 * Unauthorized entry point.
	 *
	 * @return the authentication entry point
	 */
	@Bean
	public AuthenticationEntryPoint unauthorizedEntryPoint() {
		return (request, response, authException) -> response.sendError(HttpServletResponse.SC_UNAUTHORIZED);
	}

	/**
	 * Trust resolver.
	 *
	 * @return the authentication trust resolver
	 */
	@Bean
	public AuthenticationTrustResolver trustResolver() {
		return new AuthenticationTrustResolver() {

			@Override
			public boolean isRememberMe(final Authentication authentication) {
				return false;
			}

			@Override
			public boolean isAnonymous(final Authentication authentication) {
				return false;
			}

		};
	}

	@Override
	protected void configure(HttpSecurity http) throws Exception {
		http.csrf().disable().sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS).and().authorizeRequests().antMatchers("/**").permitAll().anyRequest()
				.authenticated().and().anonymous().authenticationFilter(gatewayAuthenticationTokenFilter()).authenticationProvider(gatewayAuthenticationProvider());
	}

}
