package com.altimetrik.playground.innovation.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.altimetrik.playground.innovation.bean.PgIdeaCategoryListResponse;
import com.altimetrik.playground.innovation.bean.PgIdeaCategoryRequest;
import com.altimetrik.playground.innovation.bean.PgIdeaCategoryResponse;
import com.altimetrik.playground.innovation.bean.PgReviewer;
import com.altimetrik.playground.innovation.bean.StatusDetails;
import com.altimetrik.playground.innovation.constants.ReviewLevelEnum;
import com.altimetrik.playground.innovation.entity.PgCategoryEntity;
import com.altimetrik.playground.innovation.entity.PgReviewersEntity;
import com.altimetrik.playground.innovation.exception.InnovationServiceException;
import com.altimetrik.playground.innovation.repository.PgIdeaCategoryRepository;
import com.altimetrik.playground.innovation.service.PgIdeaCategoryService;
import com.altimetrik.playground.innovation.util.UserUtil;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class PgIdeaCategoryServiceImpl implements PgIdeaCategoryService {

	private static final String NOT_FOUND = "not.found";
	Map<Long, String> userMap = new HashMap<>();
	@Autowired
	private PgIdeaCategoryRepository pgIdeaCategoryRepository;
	@Autowired
	private UserUtil userUtil;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaCategoryService#
	 * findAllCategories()
	 */
	@Override
	public PgIdeaCategoryListResponse findAllCategories() throws InnovationServiceException {
		PgIdeaCategoryListResponse response = new PgIdeaCategoryListResponse();
		List<PgIdeaCategoryResponse> result = new ArrayList<>();
		final List<PgCategoryEntity> pgIdeaCategories = this.pgIdeaCategoryRepository.findAll();
		if (pgIdeaCategories != null)
			pgIdeaCategories.stream().forEach(pgCategoryEntity -> result.add(prepareIdeaCategoryResponse(pgCategoryEntity)));
		response.setResult(result);
		response.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return response;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaCategoryService#
	 * createCategory(
	 * com.altimetrik.playground.innovation.bean.PgIdeaCategoryResponse)
	 */
	@Override
	public PgIdeaCategoryResponse createCategory(PgIdeaCategoryRequest req) throws InnovationServiceException {
		PgIdeaCategoryResponse res;
		PgCategoryEntity bom = new PgCategoryEntity();
		List<PgReviewer> reviewers = req.getReviewers();
		List<PgReviewersEntity> rev = new ArrayList<>();
		reviewers.stream().forEach(pgReviewer -> {
			PgReviewersEntity reviewersEntity = new PgReviewersEntity(pgReviewer.getReviewerId(), ReviewLevelEnum.LEVEL_0);
			reviewersEntity.setName(pgReviewer.getReviewerName());
			rev.add(reviewersEntity);
		});
		if (!rev.isEmpty())
			bom.setReviewers(rev);
		bom.setCategoryName(req.getCategoryName());
		bom = pgIdeaCategoryRepository.save(bom);
		res = prepareIdeaCategoryResponse(bom);
		res.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return res;
	}

	private PgIdeaCategoryResponse prepareIdeaCategoryResponse(PgCategoryEntity bom) {
		PgIdeaCategoryResponse response = new PgIdeaCategoryResponse();
		response.setCategoryId(bom.getId());
		response.setCategoryName(bom.getCategoryName());
		response.setCreatedBy(bom.getCreatedBy());
		response.setCreatedDate(bom.getCreatedDate());
		List<PgReviewer> reviewerList = new ArrayList<>();
		List<PgReviewersEntity> reviewers = bom.getReviewers();
		if (reviewers != null)
			reviewers.stream().forEach(pgReviewersEntity -> {
				PgReviewer pgReviewer = new PgReviewer();
				pgReviewer.setReviewerId(pgReviewersEntity.getUserId());
				pgReviewer.setReviewerName(pgReviewersEntity.getName());
				pgReviewer.setReviewLevel(pgReviewersEntity.getLevel());
				pgReviewer.setId(pgReviewersEntity.getId());
				reviewerList.add(pgReviewer);
			});
		response.setReviewers(reviewerList);
		return response;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.altimetrik.playground.innovation.service.PgIdeaCategoryService#
	 * findByCategoryId(java. lang.String)
	 */
	@Override
	public PgIdeaCategoryResponse findByCategoryId(String id) throws InnovationServiceException {
		PgIdeaCategoryResponse res;
		Optional<PgCategoryEntity> categoryEntity = pgIdeaCategoryRepository.findById(id);
		if (!categoryEntity.isPresent())
			throw new InnovationServiceException(NOT_FOUND, "Idea Category not found");
		PgCategoryEntity cat = categoryEntity.get();
		res = prepareIdeaCategoryResponse(cat);
		res.setStatus(new StatusDetails(StatusDetails.SUCCESS));
		return res;
	}
}
