/*******************************************************************************
 * Copyright (C) Altimetrik 2018. All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Altimetrik. You shall not disclose such Confidential Information
 * and shall use it only in accordance with the terms and conditions
 * entered into with Altimetrik.
 ******************************************************************************/
package com.altimetrik.playground.innovation.webmvc;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import com.altimetrik.playground.innovation.annotation.RolesAllowed;
import com.altimetrik.playground.innovation.exception.InvalidRoleInfoException;
import com.altimetrik.playground.innovation.properties.ErrorProperties;
import com.altimetrik.playground.innovation.repository.PgIdeaRepository;
import com.sun.jersey.core.util.ReaderWriter;

import lombok.extern.slf4j.Slf4j;

/**
 * This Filter will intercept the incoming requests for web services which are
 * tagged with EnableRole annotation.
 * 
 * @author skondapalli.
 */
@Slf4j
@Component
public class RolesInterceptor extends HandlerInterceptorAdapter {

	@Autowired
	private ErrorProperties errorProperties;

	@Autowired
	private PgIdeaRepository pgIdeaRepository;

	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
		final RequestWrapper servletRequest = new RequestWrapper(request);
		if (handler instanceof HandlerMethod) {
			final HandlerMethod handlerMethod = (HandlerMethod) handler;
			final RolesAllowed rolesAllowed = this.getRolesAllowedAnnotation(handlerMethod);
			final String payload = this.getRequestBody(servletRequest);
			switch (rolesAllowed.action()) {
			case UPDATE_IDEA:
				// final PgIdeaRequestBean pgIdeaRequest = new
				// Gson().fromJson(payload, PgIdeaRequestBean.class);
				// this.updateIdeaActionValidator(pgIdeaRequest,
				// Arrays.asList(rolesAllowed.roles()));
				break;
			case UPDATE_IDEA_STATUS:
				// final PgIdeaStatusRequestBean pgIdeaStatusRequest = new
				// Gson().fromJson(payload, PgIdeaStatusRequestBean.class);
				// this.updateIdeaStatusActionValidator(pgIdeaStatusRequest);
				break;
			}
		}
		return true;
	}

	private RolesAllowed getRolesAllowedAnnotation(final HandlerMethod handlerMethod) throws InvalidRoleInfoException {
		final RolesAllowed rolesAllowed = handlerMethod.getMethod().getAnnotation(RolesAllowed.class);
		if (rolesAllowed == null) {
			throw new InvalidRoleInfoException(this.errorProperties.getInvalidRoleInfoErrorCode(), this.errorProperties.getInvalidRoleInfoErrorMessage());
		}
		return rolesAllowed;
	}

	// private void updateIdeaActionValidator(final PgIdeaRequestBean
	// pgIdeaRequest, final List<RoleTypeEnum> allowedRoles) throws
	// AuthorizationException {
	// // No id meaning not update so skip the role validation.
	// if (pgIdeaRequest.getIdeaId() != 0) {
	// final RoleTypeEnum userRole =
	// this.pgIdeaRepository.findTeamMemberRoleByIdeaIdAndEmailId(pgIdeaRequest.getIdeaId(),
	// pgIdeaRequest.getLoggedInUser());
	// if (!allowedRoles.contains(userRole)) {
	// throw new
	// AuthorizationException(this.errorProperties.getAuthorizationErrorCode(),
	// this.errorProperties.getAuthorizationErrorMessage());
	// }
	// }
	// }
	//
	// private void updateIdeaStatusActionValidator(final
	// PgIdeaStatusRequestBean pgIdeaStatusRequest) throws
	// AuthorizationException, InnovationServiceException {
	// final boolean isTeamMember =
	// (this.pgIdeaRepository.findByIdeaIdAndTeamMemberEmailId(pgIdeaStatusRequest.getIdeaId(),
	// pgIdeaStatusRequest.getLoggedInUser()) != null) ? true
	// : false;
	// if
	// (UserTypeEnum.TEAM_MEMBER.getAllowedIdeaStatusTransitions().contains(pgIdeaStatusRequest.getIdeaStatus())
	// && !isTeamMember) {
	// throw new
	// AuthorizationException(this.errorProperties.getOperationNotAllowedErrorCode(),
	// this.errorProperties.getOperationNotAllowedErrorMessage());
	// }
	// final boolean isReviewer =
	// (this.pgIdeaRepository.findByIdeaIdAndIdeaReviewerEmailId(pgIdeaStatusRequest.getIdeaId(),
	// pgIdeaStatusRequest.getLoggedInUser()) != null) ? true
	// : false;
	// if
	// (UserTypeEnum.REVIEWER.getAllowedIdeaStatusTransitions().contains(pgIdeaStatusRequest.getIdeaStatus())
	// && (!isReviewer || isTeamMember)) {
	// throw new
	// AuthorizationException(this.errorProperties.getOperationNotAllowedErrorCode(),
	// this.errorProperties.getOperationNotAllowedErrorMessage());
	// }
	// }

	private String getRequestBody(final RequestWrapper request) throws IOException {
		final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		final InputStream inputStream = request.getInputStream();
		final StringBuilder entityStream = new StringBuilder();
		try {
			ReaderWriter.writeTo(inputStream, byteArrayOutputStream);
			final byte[] entityBytes = byteArrayOutputStream.toByteArray();
			if (entityBytes.length > 0) {
				entityStream.append(new String(entityBytes));
			}
		}
		catch (final Exception exception) {
			log.error("Exception occurred while executing getRequestBody", exception);
		}
		finally {
			// TODO: handle finally clause
			// open resources will be closed by default.
			if (inputStream != null)
				inputStream.close();
		}
		return entityStream.toString();
	}

}
